@@ -13,15 +13,12 @@
 #include <kzip.h>
 
 #include <QDebug>
-#include <QTest>
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-#include <QTextCodec>
-#endif
 #include <QFileInfo>
 #include <QRegularExpression>
 #include <QSaveFile>
 #include <QStandardPaths>
 #include <QTemporaryDir>
+#include <QTest>
 #include <kcompressiondevice.h>
 
 #ifndef Q_OS_WIN
@@ -39,15 +36,6 @@
 
 QTEST_MAIN(KArchiveTest)
 
-void initLocale()
-{
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-    // KArchive uses QFile::decodeName, and our tests use utf8 encoding for filenames
-    QTextCodec::setCodecForLocale(QTextCodec::codecForName("utf-8"));
-#endif
-}
-Q_CONSTRUCTOR_FUNCTION(initLocale)
-
 static const int SIZE1 = 100;
 
 /**
