@@ -2,13 +2,13 @@
    SPDX-FileCopyrightText: 2000 David Faure <faure@kde.org>
    SPDX-FileCopyrightText: 2003 Leo Savernik <l.savernik@aon.at>
 
-   SPDX-License-Identifier: LGPL-2.0-only
+   SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
 #include "ktar.h"
 #include "karchive_p.h"
+#include "kcompressiondevice.h"
 #include "kfilterbase.h"
-#include "kfilterdev.h"
 #include "loggingcategory.h"
 
 #include <QDebug>
@@ -25,12 +25,19 @@
 ////////////////////////////////////////////////////////////////////////
 
 // Mime types of known filters
-static const char application_gzip[] = "application/x-gzip";
 static const char application_bzip[] = "application/x-bzip";
 static const char application_lzma[] = "application/x-lzma";
 static const char application_xz[] = "application/x-xz";
 static const char application_zstd[] = "application/zstd";
 
+/* clang-format off */
+namespace MimeType
+{
+QString application_gzip()     { return QStringLiteral("application/gzip"); }
+QString application_gzip_old() { return QStringLiteral("application/x-gzip"); }
+}
+/* clang-format on */
+
 class Q_DECL_HIDDEN KTar::KTarPrivate
 {
 public:
@@ -63,7 +70,15 @@ KTar::KTar(const QString &fileName, const QString &_mimetype)
     : KArchive(fileName)
     , d(new KTarPrivate(this))
 {
-    d->mimetype = _mimetype;
+    // shared-mime-info < 1.1 does not know about application/gzip.
+    // While Qt has optionally a copy of shared-mime-info (1.10 for 5.15.0),
+    // it uses the system one if it exists.
+    // Once shared-mime-info 1.1 is required or can be assumed on all targeted
+    // platforms (right now RHEL/CentOS 6 as target of appimage-based apps
+    // bundling also karchive does not meet this requirement)
+    // switch to use the new application/gzip id instead when interacting with QMimeDatabase
+    // For now: map new name to legacy name and use that
+    d->mimetype = (_mimetype == MimeType::application_gzip()) ? MimeType::application_gzip_old() : _mimetype;
 }
 
 KTar::KTar(QIODevice *dev)
@@ -97,10 +112,11 @@ bool KTar::createDevice(QIODevice::OpenMode mode)
 
         // qCDebug(KArchiveLog) << mode << mime->name();
 
-        if (mime.inherits(QStringLiteral("application/x-compressed-tar")) || mime.inherits(QString::fromLatin1(application_gzip))) {
+        if (mime.inherits(QStringLiteral("application/x-compressed-tar")) || mime.inherits(MimeType::application_gzip_old())) {
             // gzipped tar file (with possibly invalid file name), ask for gzip filter
-            d->mimetype = QString::fromLatin1(application_gzip);
-        } else if (mime.inherits(QStringLiteral("application/x-bzip-compressed-tar")) || mime.inherits(QString::fromLatin1(application_bzip))) {
+            d->mimetype = MimeType::application_gzip_old();
+        } else if (mime.inherits(QStringLiteral("application/x-bzip-compressed-tar")) || mime.inherits(QStringLiteral("application/x-bzip2-compressed-tar"))
+                   || mime.inherits(QStringLiteral("application/x-bzip2")) || mime.inherits(QString::fromLatin1(application_bzip))) {
             // bzipped2 tar file (with possibly invalid file name), ask for bz2 filter
             d->mimetype = QString::fromLatin1(application_bzip);
         } else if (mime.inherits(QStringLiteral("application/x-lzma-compressed-tar")) || mime.inherits(QString::fromLatin1(application_lzma))) {
@@ -123,8 +139,8 @@ bool KTar::createDevice(QIODevice::OpenMode mode)
         }
         if (!d->mimetype.isEmpty()) {
             // Create a compression filter on top of the QSaveFile device that KArchive created.
-            // qCDebug(KArchiveLog) << "creating KFilterDev for" << d->mimetype;
-            KCompressionDevice::CompressionType type = KFilterDev::compressionTypeForMimeType(d->mimetype);
+            // qCDebug(KArchiveLog) << "creating KCompressionDevice for" << d->mimetype;
+            KCompressionDevice::CompressionType type = KCompressionDevice::compressionTypeForMimeType(d->mimetype);
             d->compressionDevice = new KCompressionDevice(device(), false, type);
             setDevice(d->compressionDevice);
         }
@@ -134,9 +150,9 @@ bool KTar::createDevice(QIODevice::OpenMode mode)
         // So instead of applying the filter to the device,
         // the file is completely extracted instead,
         // and we work on the extracted tar file.
-        // This improves the extraction speed by the tar ioslave dramatically,
+        // This improves the extraction speed by the archive KIO worker supporting the tar protocol dramatically,
         // if the archive file contains many files.
-        // This is because the tar ioslave extracts one file after the other and normally
+        // This is because the archive KIO worker extracts one file after the other and normally
         // has to walk through the decompression filter each time.
         // Which is in fact nearly as slow as a complete decompression for each file.
 
@@ -315,7 +331,7 @@ bool KTar::KTarPrivate::fillTempFile(const QString &fileName)
 
     // qCDebug(KArchiveLog) << "filling tmpFile of mimetype" << mimetype;
 
-    KCompressionDevice::CompressionType compressionType = KFilterDev::compressionTypeForMimeType(mimetype);
+    KCompressionDevice::CompressionType compressionType = KCompressionDevice::compressionTypeForMimeType(mimetype);
     KCompressionDevice filterDev(fileName, compressionType);
 
     QFile *file = tmpFile;
@@ -453,7 +469,7 @@ bool KTar::openArchive(QIODevice::OpenMode mode)
             // == '2' );
 
             if (typeflag == 'x' || typeflag == 'g') { // pax extended header, or pax global extended header
-                // Skip it for now. TODO: implement reading of extended header, as per http://pubs.opengroup.org/onlinepubs/009695399/utilities/pax.html
+                // Skip it for now. TODO: implement reading of extended header, as per https://pubs.opengroup.org/onlinepubs/9699919799/utilities/pax.html
                 (void)dev->read(buffer, 0x200);
                 continue;
             }
@@ -546,8 +562,12 @@ bool KTar::KTarPrivate::writeBackTempFile(const QString &fileName)
     // qCDebug(KArchiveLog) << "Write temporary file to compressed file" << fileName << mimetype;
 
     bool forced = false;
-    if (QLatin1String(application_gzip) == mimetype || QLatin1String(application_bzip) == mimetype || QLatin1String(application_lzma) == mimetype
-        || QLatin1String(application_xz) == mimetype) {
+    /* clang-format off */
+    if (MimeType::application_gzip_old() == mimetype ||
+        QLatin1String(application_bzip) == mimetype ||
+        QLatin1String(application_lzma) == mimetype ||
+        QLatin1String(application_xz) == mimetype) {
+        /* clang-format on */
         forced = true;
     }
 
@@ -555,7 +575,7 @@ bool KTar::KTarPrivate::writeBackTempFile(const QString &fileName)
     // (KArchive uses QSaveFile by default, but the temp-uncompressed-file trick
     // circumvents that).
 
-    KFilterDev dev(fileName);
+    KCompressionDevice dev(fileName);
     QFile *file = tmpFile;
     if (!dev.open(QIODevice::WriteOnly)) {
         file->close();
@@ -749,6 +769,12 @@ bool KTar::doPrepareWriting(const QString &name,
         return false;
     }
 
+    const qint64 MAX_FILESIZE = 077777777777L; // the format we use only allows 11 octal digits for size
+    if (size > MAX_FILESIZE) {
+        setErrorString(tr("Application limitation: Can not add file larger than %1 bytes").arg(MAX_FILESIZE));
+        return false;
+    }
+
     // In some tar files we can find dir/./file => call cleanPath
     QString fileName(QDir::cleanPath(name));
 
@@ -771,8 +797,8 @@ bool KTar::doPrepareWriting(const QString &name,
       }
     */
 
-    char buffer[0x201];
-    memset(buffer, 0, 0x200);
+    char buffer[0x201] = {0};
+
     if ((mode() & QIODevice::ReadWrite) == QIODevice::ReadWrite) {
         device()->seek(d->tarEnd); // Go to end of archive as might have moved with a read
     }
@@ -838,8 +864,8 @@ bool KTar::doWriteDir(const QString &name,
         return true; // already there
     }
 
-    char buffer[0x201];
-    memset(buffer, 0, 0x200);
+    char buffer[0x201] = {0};
+
     if ((mode() & QIODevice::ReadWrite) == QIODevice::ReadWrite) {
         device()->seek(d->tarEnd); // Go to end of archive as might have moved with a read
     }
@@ -898,8 +924,8 @@ bool KTar::doWriteSymLink(const QString &name,
     // In some tar files we can find dir/./file => call cleanPath
     QString fileName(QDir::cleanPath(name));
 
-    char buffer[0x201];
-    memset(buffer, 0, 0x200);
+    char buffer[0x201] = {0};
+
     if ((mode() & QIODevice::ReadWrite) == QIODevice::ReadWrite) {
         device()->seek(d->tarEnd); // Go to end of archive as might have moved with a read
     }
